@@ -1,15 +1,8 @@
 #!/usr/bin/python3
 
-class Error(Exception):
+class NotAValidOption(Exception):
     """
-    Base Class for all the Exceptions
-    """
-
-    pass
-
-class NotAValidOption(Error):
-    """
-    Checking for a valid option
+    Checking for a valid list option
     """
 
     def __init__(self, option, valid_options):
@@ -18,4 +11,14 @@ class NotAValidOption(Error):
 
         self.message = self.option+" is not a valid option, "+str(self.valid_options)+" are the only valid options"
 
+        super().__init__(self.message)
+
+class CannotCreateModel(Exception):
+    """
+    Cannot Create a Model
+    """
+
+    def __init__(self, message):
+        self.message = message
+
         super().__init__(self.message)
\ No newline at end of file
